# -*- coding: utf-8 -*-
from setuphelpers import *
import waptguihelper
import subprocess


def update_package():
    # Declaring local variables
    package_updated = False
    iso_path = makepath(basedir, "iso")
    setup_path = makepath(iso_path, "setup.exe")
    iso_present = False
    iso_extracted = False
    iso_copy_path = None
    iso_to_download = False
    keep_extracted_iso = False
    keep_current_iso = False

    lang_dict = {
        "fr": "French",
        "en": "English",
        "pl": "Polish",
        "de": "German",
        "es": "Spanish",
        "pt": "Portuguese",
        "it": "Italian",
        "nl": "Dutch",
        "ru": "Russian",
        "all": "Others",
    }
    lang_list = []
    for entry in lang_dict:
        lang_list.append({"short name": entry, "long name": lang_dict[entry]})

    # Detect ISO if placed
    for iso_in_dir in glob.glob("*.iso"):
        if isfile(iso_in_dir):
            iso_present = True

    # Detect ISO if already extracted
    if isfile(setup_path):
        iso_extracted = True

    if iso_present:
        keep_iso_present_question = waptguihelper.message_dialog("ISO found", f"Do you want to keep: {iso_in_dir}?", waptguihelper.ID_YES)
        if keep_iso_present_question == waptguihelper.ID_YES:
            keep_current_iso = True

    if iso_extracted:
        keep_iso_extracted_question = waptguihelper.message_dialog("An ISO is already extracted", "Do you want to keep it?", waptguihelper.ID_YES)
        if keep_iso_extracted_question == waptguihelper.ID_YES:
            keep_extracted_iso = True

    if keep_current_iso and keep_extracted_iso:
        keep_only_one_question = waptguihelper.grid_dialog(
            "Please select only one", [{"keep :": iso_in_dir}, {"keep :": "iso already extracted"}], waptguihelper.GRT_SELECTED
        )
        if "extracted" in keep_only_one_question[0]["keep :"]:
            keep_current_iso = False
        else:
            keep_extracted_iso = False

    if not keep_extracted_iso:
        if not iso_present:
            iso_absent_question = waptguihelper.message_dialog(
                "No ISO detected", "Do you want to download Windows ISO from Microsoft's servers?\nClick No to use your own ISO", waptguihelper.ID_YES
            )
            if iso_absent_question == waptguihelper.ID_YES:
                iso_to_download = True

    if iso_to_download:
        # Download from browser
        try:
            if isdir(makepath(programfiles32, "Google", "Chrome", "Application")):
                chrome_app_path = makepath(programfiles32, "Google", "Chrome", "Application")
            else:
                chrome_app_path = makepath(programfiles, "Google", "Chrome", "Application")
            chrome_bin_path = makepath(chrome_app_path, "chrome.exe")
            user_agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36"
            cmd_browser = rf'"{chrome_bin_path}" --user-agent="{user_agent}" --user-data-dir=%tmp%\chrome_tmp "{"https://www.microsoft.com/software-download/windows10"}"'
            waptguihelper.message_dialog(
                "Chrome will open with a custom user-agent",
                'Select "Download Windows 10 Disk Image (ISO)" => "Windows 10 (multi-edition ISO) => then your language. When the download is finished, please close the opened Chrome',
            )
            subprocess.Popen(cmd_browser, shell=True, start_new_session=True)
            waptguihelper.message_dialog("Waiting for download", "Click OK when the download is complete", waptguihelper.ID_OK)
        except:
            error(cmd_browser)

    # Copy to pkg
    if not keep_current_iso and not iso_present and not keep_extracted_iso:
        # Cleanup ISO files
        for iso_in_dir in glob.glob("*.iso"):
            remove_file(iso_in_dir)

        iso_copy_path = waptguihelper.filename_dialog("Please provide the Windows ISO to copy in this package", "", "", "ISO Files|*.iso")
        print("Copying: " + iso_copy_path)
        filecopyto(iso_copy_path, basedir)

    # Extracting ISO
    if not keep_extracted_iso:
        # Cleanup ISO dir
        if isdir(iso_path):
            remove_tree(iso_path)
        iso_in_dir = glob.glob("*.iso")[0]
        print("Extracting: " + iso_in_dir)
        unzip_with_7zip(iso_in_dir, iso_path)
        remove_file(iso_in_dir)

    # Getting version from the ISO setup.exe
    version = ".".join(get_file_properties(setup_path)["ProductVersion"].split(".")[:3])
    if version == "10.0.19041":
        version = waptguihelper.input_dialog(control.package, "Please provide the correct version number of this ISO", version)

    # Changing version of the package
    if Version(version) > control.get_software_version():
        print("Software version updated from: %s to: %s" % (control.get_software_version(), Version(version)))
        package_updated = True
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Changing locale of the package
    lang_short = waptguihelper.grid_dialog("Please select a language", lang_list, waptguihelper.GRT_SELECTED)[0]["short name"]

    # Updating pkg infos
    if "-template" in control.package:
        package_name = waptguihelper.input_dialog(control.package, "You can redefine the package name", control.package.split("-template")[0])
        control.package = package_name
        if " Template" in control.name:
            control.name = control.name.split(" Template")[0]
        print("Changing package name to: %s" % package_name)
    if "all" in control.architecture:
        package_architecture = waptguihelper.combo_dialog(control.package, "Select architecture", ["x64", "x86"])
        control.architecture = package_architecture
        print("Changing arch to: %s" % package_architecture)
    control.locale = lang_short
    control.save_control_to_wapt()
    print("Changing locale to: %s in WAPT\\control" % control.locale)

    # Validating update-package-sources
    return package_updated
